from policy import allowed_tags_attributes as ata
import sys
import lxml
from lxml import etree

DEFAULT_ENCODING='utf8'

def filter(tree):
    """a DFS striping unauthorized nodes and attributes."""
    newNode=lxml.etree.Element(tree.tag)
    
    #nothing to do
    if tree.tag not in ata:
        return None
   
    #filter allowed attributes
    for attr in tree.attrib:        
        if attr in ata[tree.tag]:
            newNode.attrib[attr]=tree.attrib[attr]

    #always allow text
    newNode.text=tree.text
    newNode.tail=tree.tail

    #recurse on subtree, adding trailing text
    for e in tree:        
        if e.tag in ata:
            newNode.append(filter(e))
        else:
            if e.tail:
                if newNode.tail:
                    newNode.tail+=' '+e.tail
                else:
                    newNode.tail=e.tail
    return newNode

def __makeReferences(tree, counter):
    """a DFS returning a reference list and a modified tree."""
    refs=[]
    
    if tree.tag == 'a':
        refs.append([counter, tree.attrib.get('href', '')])
        tree.tag='span'
        for x in tree.attrib:
            del tree.attrib[x]
        if tree.text:
            tree.text+='['+str(counter)+']'
        else:
            tree.text='['+str(counter)+']'
        counter+=1

    if tree.tag == 'img':
        if not tree.attrib.get('src', '')[:10]=='data:image':
            refs.append([counter, tree.attrib.get('src', '')])
            tree.tag='span'
            for x in tree.attrib:
                del tree.attrib[x]
            tree.text='IMAGE['+str(counter)+']'
            counter+=1
    
    for e in tree:        
        r, counter = __makeReferences(e, counter)
        refs.extend(r)
    return refs, counter

makeReferences = lambda tree: __makeReferences(tree, 0)[0]

def insertReferences(tree, references):
    text=["---------------------------\nReferences:\n\n"]
    for num, ref in references:
        text.extend(["[",str(num),"]: " , ref.decode(DEFAULT_ENCODING, 'replace'), '\n']) 
    e=tree.find("body")
    if e!=None and len(e)>0:
        pre=etree.SubElement(e ,'pre')
        pre.text=''.join(text)

def process(tree):
    clean = filter(tree)
    refs=makeReferences(clean)
    b=tree.find(".//base")
    if b!=None and len(b)>0:
        if len(b.attrib)>0 and 'href' in b.attrib:
            base=b.attrib['href']
    else:
        base=''
    for el in refs:
        el[1]=base+el[1]
    insertReferences(clean, refs)
    return clean
